@@ -93,18 +93,30 @@ def prune_ast(json, parent=None):
     if isinstance(json, dict):
         for k, v in json.items():
             child = prune_ast(json=v, parent=k)
-            if child:
+            if child is not None:
                 pruned[k] = child
         pruned = validate_tree(k=parent, json=pruned)
+        if pruned == {} and pruned != json:
+            pruned = None
     elif isinstance(json, list):
+        child_parent = None
+        if parent:
+            if parent == 'from':
+                child_parent = 'from'
+            elif parent == 'cross join':
+                child_parent = 'cross join'
+            elif 'join' in parent:
+                child_parent = 'join'
+            elif 'where' == 'parent':
+                child_parent = 'where'
         for v in json:
-            child = prune_ast(json=v, parent=None)
-            if child:
+            child = prune_ast(json=v, parent=child_parent)
+            if child is not None:
                 pruned.append(child)
         pruned = validate_tree(k=parent, json=pruned)
-    else:
-        if json is not None:
-            pruned = json
+    elif json is not None:
+        pruned = json
+
     return pruned
 
 
@@ -115,8 +127,14 @@ def validate_tree(k, json):
     :param json: the query AST
     :return: None if invalid and the query_ast if valid.
     """
-    from camtono.parser.parse import min_keys
-    if k is None or len(json) >= min_keys.get(k, 1):
+    from camtono.parser.keywords import min_keys
+    if k is None:
+        return json
+    elif isinstance(json, dict) and any(set(json.keys()) >= set(i) for i in min_keys.get(k, [list(json.keys())])):
+        return json
+    elif isinstance(json, list) and len(json) >= len(min_keys.get(k, [['value']])[0]):
+        return json
+    elif any(isinstance(json, i) for i in [str, int, float, bool]):
         return json
     else:
         return None
