@@ -75,43 +75,60 @@ reserved = [
 ]
 
 unions = [UNION_DISTINCT, UNION_ALL, UNION]
-
+joins = ['left', 'inner', 'outer', 'cross', 'full outer', 'right', 'right inner', 'right outer', 'left inner',
+         'left outer']
 min_keys = {
-    "cast": 2,
-    "mul": 2,
-    "div": 2,
-    "mod": 2,
-    "neg": 2,
-    "add": 2,
-    "sub": 2,
-    "case": 2,
-    "binary_not": 2,
-    "binary_and": 2,
-    "binary_or": 2,
-    "timestamp_sub": 2,
-    "timestamp_add": 2,
-    "date_add": 2,
-    "date_sub": 2,
-    "datetime_sub": 2,
-    "datetime_add": 2,
-    "gte": 2,
-    "lte": 2,
-    "lt": 2,
-    "gt": 2,
-    "eq": 2,
-    "neq": 2,
-    "between": 2,
-    "not_between": 2,
-    "interval": 2,
-    "in": 2,
-    "nin": 2,
-    "is": 2,
-    "like": 2,
-    "not_like": 2,
-    "rlike": 2,
-    "not_rlike": 2,
-    "similar_to": 2,
-    "not_similar_to": 2,
+    "cast": [['value', 'type']],
+    "mul": [['a', 'b']],
+    "div": [['a', 'b']],
+    "mod": [['a', 'b']],
+    "neg": [['a', 'b']],
+    "add": [['a', 'b']],
+    "sub": [['a', 'b']],
+    "case": [['when', 'then']],
+    "binary_not": [['a', 'b']],
+    "binary_and": [['a', 'b']],
+    "binary_or": [['a', 'b']],
+    "timestamp_sub": [['a', 'b']],
+    "timestamp_add": [['a', 'b']],
+    "date_add": [['a', 'b']],
+    "date_sub": [['a', 'b']],
+    "datetime_sub": [['a', 'b']],
+    "datetime_add": [['a', 'b']],
+    "from": [['value'], ['join', 'using'], ['join', 'on'], ['left join', 'using'], ['left join', 'on'], ['value'],
+             ['inner join', 'using'], ['inner join', 'on'], ['outer join', 'using'], ['outer join', 'on'],
+             ['full outer join', 'using'], ['full outer join', 'on'], ['full join', 'using'], ['full join', 'on']
+             ],
+    "on": [['eq'], ['or'], ['and']],
+    "gte": [['a', 'b']],
+    "lte": [['a', 'b']],
+    "join": [['value']],
+    "lt": [['a', 'b']],
+    "gt": [['a', 'b']],
+    "eq": [['a', 'b']],
+    "neq": [['a', 'b']],
+    "between": [['a', 'b']],
+    "not_between": [['a', 'b']],
+    "interval": [['a', 'b']],
+    "in": [['a', 'b']],
+    "nin": [['a', 'b']],
+    "is": [['a', 'b']],
+    "like": [['a', 'b']],
+    "not_like": [['a', 'b']],
+    "rlike": [['a', 'b']],
+    "not_rlike": [['a', 'b']],
+    "similar_to": [['a', 'b']],
+    "not_similar_to": [['a', 'b']],
+    "where": [['gte'], ['lte'], ['lt'], ['gt'], ['eq'], ['neq'], ['between'], ['not_between'], ['interval'], ['in'],
+              ['nin'], ['is'], ['like'], ['not_like'], ['rlike'], ['not_rlike'], ['similar_to'], ['not_similar_to'],
+              ['missing'], ['and'], ['or']],
+    "and": [['gte'], ['lte'], ['lt'], ['gt'], ['eq'], ['neq'], ['between'], ['not_between'], ['interval'], ['in'],
+            ['nin'], ['is'], ['like'], ['not_like'], ['rlike'], ['not_rlike'], ['similar_to'], ['not_similar_to'],
+            ['missing'], ['and'], ['or']],
+    "or": [['gte'], ['lte'], ['lt'], ['gt'], ['eq'], ['neq'], ['between'], ['not_between'], ['interval'], ['in'],
+           ['nin'], ['is'], ['like'], ['not_like'], ['rlike'], ['not_rlike'], ['similar_to'], ['not_similar_to'],
+           ['missing'], ['and'], ['or']],
+    **{"{} join".format(i): [['value']] for i in joins}
 }
 
 durations = {
